@@ -1,10 +1,16 @@
 import copy
-from operator import add
 import types
 
-from .vendor import six
-from .vendor.lexicon import Lexicon
+try:
+    from .vendor import six
+    from .vendor.lexicon import Lexicon
+except ImportError:
+    # Use system wide modules
+    import six
+    from lexicon import Lexicon
 
+
+from .config import merge_dicts, copy_dict
 from .parser import Context as ParserContext
 from .tasks import Task
 
@@ -20,6 +26,16 @@ class Collection(object):
         `.Collection` offers a set of methods for building a collection of
         tasks from scratch, plus a convenient constructor wrapping said API.
 
+        In either case:
+
+        * the first positional argument may be a string, which (if given) is
+          used as the collection's default name when performing namespace
+          lookups;
+        * a ``loaded_from`` keyword argument may be given, which sets metadata
+          indicating the filesystem path the collection was loaded from. This
+          is used as a guide when loading per-project :ref:`configuration files
+          <config-hierarchy>`.
+
         **The method approach**
 
         May initialize with no arguments and use methods (e.g.
@@ -45,12 +61,12 @@ class Collection(object):
 
         **The constructor approach**
 
-        All ``*args`` given to `.Collection` (besides the optional first 'name'
-        argument) are expected to be `.Task` or `.Collection` instances which
-        will be passed to `.add_task`/`.add_collection` as appropriate. Module
-        objects are also valid (as they are for `.add_collection`). For
-        example, the below snippet results in the same two task identifiers as
-        the one above::
+        All ``*args`` given to `.Collection` (besides the abovementioned
+        optional positional 'name' argument and ``loaded_from`` kwarg) are
+        expected to be `.Task` or `.Collection` instances which will be passed
+        to `.add_task`/`.add_collection` as appropriate. Module objects are
+        also valid (as they are for `.add_collection`). For example, the below
+        snippet results in the same two task identifiers as the one above::
 
             ns = Collection(top_level_task, Collection('docs', doc_task))
 
@@ -81,6 +97,8 @@ class Collection(object):
         args = list(args)
         if args and isinstance(args[0], six.string_types):
             self.name = args.pop(0)
+        # Specific kwargs if applicable
+        self.loaded_from = kwargs.pop('loaded_from', None)
         # Dispatch args/kwargs
         for arg in args:
             self._add_object(arg)
@@ -94,7 +112,7 @@ class Collection(object):
         elif isinstance(obj, (Collection, types.ModuleType)):
             method = self.add_collection
         else:
-            raise TypeError("No idea how to insert %r!" % type(obj))
+            raise TypeError("No idea how to insert {0!r}!".format(type(obj)))
         return method(obj, name=name)
 
     def __str__(self):
@@ -108,7 +126,7 @@ class Collection(object):
         return self.name == other.name and self.tasks == other.tasks
 
     @classmethod
-    def from_module(self, module, name=None, config=None):
+    def from_module(self, module, name=None, config=None, loaded_from=None):
         """
         Return a new `.Collection` created from ``module``.
 
@@ -125,18 +143,22 @@ class Collection(object):
         Explicitly given collections will only be given that module-derived
         name if they don't already have a valid ``.name`` attribute.
 
-        :param name:
+        :param str name:
             A string, which if given will override any automatically derived
             collection name (or name set on the module's root namespace, if it
             has one.)
 
-        :param config:
-            A dict, used to set config options on the newly created
-            `.Collection` before returning it (saving you a call to
-            `.configure`.)
-            
+        :param dict config:
+            Used to set config options on the newly created `.Collection`
+            before returning it (saving you a call to `.configure`.)
+
             If the imported module had a root namespace object, ``config`` is
             merged on top of it (i.e. overriding any conflicts.)
+
+        :param str loaded_from:
+            Identical to the same-named kwarg from the regular class
+            constructor - should be the path where the module was
+            found.
         """
         module_name = module.__name__.split('.')[-1]
         # See if the module provides a default NS to use in lieu of creating
@@ -144,16 +166,18 @@ class Collection(object):
         for candidate in ('ns', 'namespace'):
             obj = getattr(module, candidate, None)
             if obj and isinstance(obj, Collection):
+                # TODO: make this into Collection.clone() or similar
                 # Explicitly given name wins over root ns name which wins over
                 # actual module name.
-                ret = Collection(name or obj.name or module_name)
+                ret = Collection(name or obj.name or module_name,
+                                 loaded_from=loaded_from)
                 ret.tasks = copy.deepcopy(obj.tasks)
                 ret.collections = copy.deepcopy(obj.collections)
                 ret.default = copy.deepcopy(obj.default)
                 # Explicitly given config wins over root ns config
-                obj_config = copy.deepcopy(obj._configuration)
+                obj_config = copy_dict(obj._configuration)
                 if config:
-                    obj_config.update(config)
+                    merge_dicts(obj_config, config)
                 ret._configuration = obj_config
                 return ret
         # Failing that, make our own collection from the module's tasks.
@@ -162,7 +186,7 @@ class Collection(object):
             vars(module).values()
         )
         # Again, explicit name wins over implicit one from module path
-        collection = Collection(name or module_name)
+        collection = Collection(name or module_name, loaded_from=loaded_from)
         for task in tasks:
             collection.add_task(task)
         if config:
@@ -192,17 +216,26 @@ class Collection(object):
             else:
                 raise ValueError("Could not obtain a name for this task!")
         if name in self.collections:
-            raise ValueError("Name conflict: this collection has a sub-collection named %r already" % name)
+            raise ValueError("Name conflict: this collection has a sub-collection named {0!r} already".format(name)) # noqa
         self.tasks[name] = task
         for alias in task.aliases:
             self.tasks.alias(alias, to=name)
         if default is True or (default is None and task.is_default):
             if self.default:
-                msg = "'%s' cannot be the default because '%s' already is!"
-                raise ValueError(msg % (name, self.default))
+                msg = "'{0}' cannot be the default because '{1}' already is!"
+                raise ValueError(msg.format(name, self.default))
             self.default = name
 
     def add_collection(self, coll, name=None):
+        """
+        Add `.Collection` ``coll`` as a sub-collection of this one.
+
+        :param coll: The `.Collection` to add.
+
+        :param str name:
+            The name to attach the collection as. Defaults to the collection's
+            own internal name.
+        """
         # Handle module-as-collection
         if isinstance(coll, types.ModuleType):
             coll = Collection.from_module(coll)
@@ -212,7 +245,7 @@ class Collection(object):
             raise ValueError("Non-root collections must have a name!")
         # Test for conflict
         if name in self.tasks:
-            raise ValueError("Name conflict: this collection has a task named %r already" % name)
+            raise ValueError("Name conflict: this collection has a task named {0!r} already".format(name)) # noqa
         # Insert
         self.collections[name] = coll
 
@@ -255,12 +288,12 @@ class Collection(object):
 
         E.g. in a deeply nested tree, this method returns the `.Task`, and a
         configuration dict created by merging that of this `.Collection` and
-        any nested `.Collections`, up through the one actually holding the
-        `.Task`.
+        any nested `Collections <.Collection>`, up through the one actually
+        holding the `.Task`.
 
-        See `__getitem__` for semantics of the ``name`` argument.
+        See `~.Collection.__getitem__` for semantics of the ``name`` argument.
 
-        :return: Two-tuple of (`.Task`, `dict`).
+        :returns: Two-tuple of (`.Task`, `dict`).
         """
         # Our top level configuration
         ours = self.configuration()
@@ -282,7 +315,7 @@ class Collection(object):
 
     def __contains__(self, name):
         try:
-            task = self[name]
+            self[name]
             return True
         except KeyError:
             return False
@@ -300,7 +333,7 @@ class Collection(object):
         return result
 
     def subtask_name(self, collection_name, task_name):
-        return "%s.%s" % (collection_name, task_name)
+        return '.'.join([collection_name, task_name])
 
     @property
     def task_names(self):
@@ -334,27 +367,25 @@ class Collection(object):
         """
         Obtain merged configuration values from collection & children.
 
-        .. note::
-            Merging uses ``copy.deepcopy`` to prevent state bleed.
-
         :param taskpath:
-            (Optional) Task name/path, identical to that used for `__getitem__`
-            (e.g. may be dotted for nested tasks, etc.) Used to decide which
-            path to follow in the collection tree when merging config values.
+            (Optional) Task name/path, identical to that used for
+            `~.Collection.__getitem__` (e.g. may be dotted for nested tasks,
+            etc.) Used to decide which path to follow in the collection tree
+            when merging config values.
 
         :returns: A `dict` containing configuration values.
         """
         if taskpath is None:
-            return copy.deepcopy(self._configuration)
+            return copy_dict(self._configuration)
         return self.task_with_config(taskpath)[1]
 
     def configure(self, options):
         """
-        Merge ``options`` dict into this collection's `.configuration`.
+        (Recursively) merge ``options`` into the current `.configuration`.
 
-        Options configured this way will be available to all
-        :doc:`contextualized tasks </concepts/context>`. It is recommended to
-        use unique keys to avoid potential clashes with other config options
+        Options configured this way will be available to all tasks. It is
+        recommended to use unique keys to avoid potential clashes with other
+        config options
 
         For example, if you were configuring a Sphinx docs build target
         directory, it's better to use a key like ``'sphinx.target'`` than
@@ -363,4 +394,4 @@ class Collection(object):
         :param options: An object implementing the dictionary protocol.
         :returns: ``None``.
         """
-        self._configuration.update(options)
+        merge_dicts(self._configuration, options)
