@@ -1,10 +1,18 @@
-from ..vendor.lexicon import Lexicon
+import itertools
+
+try:
+    from ..vendor.lexicon import Lexicon
+except ImportError:
+    from lexicon import Lexicon
 
 from .argument import Argument
 
 
+def translate_underscores(name):
+    return name.lstrip('_').rstrip('_').replace('_', '-')
+
 def to_flag(name):
-    name = name.replace('_', '-')
+    name = translate_underscores(name)
     if len(name) == 1:
         return '-' + name
     return '--' + name
@@ -40,7 +48,9 @@ def flag_key(x):
     return ret
 
 
-class Context(object):
+# Named slightly more verbose so Sphinx references can be unambiguous.
+# Got real sick of fully qualified paths.
+class ParserContext(object):
     """
     Parsing context with knowledge of flags & their format.
 
@@ -51,11 +61,11 @@ class Context(object):
     """
     def __init__(self, name=None, aliases=(), args=()):
         """
-        Create a new ``Context`` named ``name``, with ``aliases``.
+        Create a new ``ParserContext`` named ``name``, with ``aliases``.
 
         ``name`` is optional, and should be a string if given. It's used to
-        tell Context objects apart, and for use in a Parser when determining
-        what chunk of input might belong to a given Context.
+        tell ParserContext objects apart, and for use in a Parser when
+        determining what chunk of input might belong to a given ParserContext.
 
         ``aliases`` is also optional and should be an iterable containing
         strings. Parsing will honor any aliases when trying to "find" a given
@@ -74,10 +84,12 @@ class Context(object):
             self.add_arg(arg)
 
     def __str__(self):
-        aliases = (" (%s)" % ', '.join(self.aliases)) if self.aliases else ""
-        name = (" %r%s" % (self.name, aliases)) if self.name else ""
-        args = (": %r" % (self.args,)) if self.args else ""
-        return "<Context%s%s>" % (name, args)
+        aliases = ""
+        if self.aliases:
+            aliases = " ({0})".format(', '.join(self.aliases))
+        name = (" {0!r}{1}".format(self.name, aliases)) if self.name else ""
+        args = (": {0!r}".format(self.args)) if self.args else ""
+        return "<parser/Context{0}{1}>".format(name, args)
 
     def __repr__(self):
         return str(self)
@@ -105,8 +117,8 @@ class Context(object):
         # Uniqueness constraint: no name collisions
         for name in arg.names:
             if name in self.args:
-                msg = "Tried to add an argument named %r but one already exists!"
-                raise ValueError(msg % name)
+                msg = "Tried to add an argument named {0!r} but one already exists!" # noqa
+                raise ValueError(msg.format(name))
         # First name used as "main" name for purposes of aliasing
         main = arg.names[0] # NOT arg.name
         self.args[main] = arg
@@ -122,45 +134,66 @@ class Context(object):
         if arg.attr_name:
             self.args.alias(arg.attr_name, to=main)
         # Add to inverse_flags if required
-        if arg.kind == bool and arg.default == True:
+        if arg.kind == bool and arg.default is True:
             # Invert the 'main' flag name here, which will be a dashed version
             # of the primary argument name if underscore-to-dash transformation
             # occurred.
-            inverse_name = to_flag("no-%s" % main)
+            inverse_name = to_flag("no-{0}".format(main))
             self.inverse_flags[inverse_name] = to_flag(main)
 
     @property
     def needs_positional_arg(self):
         return any(x.value is None for x in self.positional_args)
 
+    @property
+    def as_kwargs(self):
+        """
+        This context's arguments' values keyed by their ``.name`` attribute.
+
+        Results in a dict suitable for use in Python contexts, where e.g. an
+        arg named ``foo-bar`` becomes accessible as ``foo_bar``.
+        """
+        ret = {}
+        for arg in self.args.values():
+            ret[arg.name] = arg.value
+        return ret
+
+    def names_for(self, flag):
+        # TODO: should probably be a method on Lexicon/AliasDict
+        return list(set([flag] + self.flags.aliases_of(flag)))
+
     def help_for(self, flag):
         """
         Return 2-tuple of ``(flag-spec, help-string)`` for given ``flag``.
         """
         # Obtain arg obj
         if flag not in self.flags:
-            raise ValueError("%r is not a valid flag for this context! Valid flags are: %r" % (flag, self.flags.keys()))
+            err = "{0!r} is not a valid flag for this context! Valid flags are: {1!r}" # noqa
+            raise ValueError(err.format(flag, self.flags.keys()))
         arg = self.flags[flag]
-        # Show all potential names for this flag in the output
-        names = list(set([flag] + self.flags.aliases_of(flag)))
         # Determine expected value type, if any
         value = {
             str: 'STRING',
         }.get(arg.kind)
         # Format & go
         full_names = []
-        for name in names:
+        for name in self.names_for(flag):
             if value:
                 # Short flags are -f VAL, long are --foo=VAL
                 # When optional, also, -f [VAL] and --foo[=VAL]
                 if len(name.strip('-')) == 1:
-                    value_ = ("[%s]" % value) if arg.optional else value
-                    valuestr = " %s" % value_
+                    value_ = ("[{0}]".format(value)) if arg.optional else value
+                    valuestr = " {0}".format(value_)
                 else:
-                    valuestr = "=%s" % value
+                    valuestr = "={0}".format(value)
                     if arg.optional:
-                        valuestr = "[%s]" % valuestr
+                        valuestr = "[{0}]".format(valuestr)
             else:
+                # no value => boolean
+                # check for inverse
+                if name in self.inverse_flags.values():
+                    name = "--[no-]{0}".format(name[2:])
+
                 valuestr = ""
             # Tack together
             full_names.append(name + valuestr)
@@ -198,3 +231,16 @@ class Context(object):
             lambda x: self.help_for(to_flag(x.name)),
             sorted(self.flags.values(), key=flag_key)
         ))
+
+    def flag_names(self):
+        """
+        Similar to `help_tuples` but returns flag names only, no helpstrs.
+
+        Specifically, all flag names, flattened, in rough order.
+        """
+        # Regular flag names
+        flags = sorted(self.flags.values(), key=flag_key)
+        names = [self.names_for(to_flag(x.name)) for x in flags]
+        # Inverse flag names sold separately
+        names.append(self.inverse_flags.keys())
+        return tuple(itertools.chain.from_iterable(names))
