@@ -2,14 +2,18 @@
 This module contains the core `.Task` class & convenience decorators used to
 generate new tasks.
 """
+
+from copy import deepcopy
 import inspect
 import types
 
-from .vendor import six
-from .vendor.lexicon import Lexicon
+try:
+    from .vendor import six
+except ImportError:
+    import six
 
 from .context import Context
-from .parser import Argument
+from .parser import Argument, translate_underscores
 
 if six.PY3:
     from itertools import zip_longest
@@ -17,7 +21,7 @@ else:
     from itertools import izip_longest as zip_longest
 
 
-# Non-None sentinel
+#: Sentinel object representing a truly blank value (vs ``None``).
 NO_DEFAULT = object()
 
 
@@ -33,7 +37,6 @@ class Task(object):
     def __init__(self,
         body,
         name=None,
-        contextualized=False,
         aliases=(),
         positional=None,
         optional=(),
@@ -42,14 +45,13 @@ class Task(object):
         help=None,
         pre=None,
         post=None,
+        autoprint=False,
     ):
         # Real callable
         self.body = body
         # Must copy doc/name here because Sphinx is stupid about properties.
         self.__doc__ = getattr(body, '__doc__', '')
         self.__name__ = getattr(body, '__name__', '')
-        # Is this a contextualized task?
-        self.contextualized = contextualized
         # Default name, alternate names, and whether it should act as the
         # default for its parent collection
         self._name = name
@@ -64,6 +66,8 @@ class Task(object):
         self.pre = pre or []
         self.post = post or []
         self.times_called = 0
+        # Whether to print return value post-execution
+        self.autoprint = autoprint
 
     @property
     def name(self):
@@ -102,9 +106,11 @@ class Task(object):
         return hash(self.name) + hash(self.body)
 
     def __call__(self, *args, **kwargs):
-        # Guard against calling contextualized tasks with no context.
-        if self.contextualized and not isinstance(args[0], Context):
-            raise TypeError("Contextualized task expected a Context, got %s instead!" % type(args[0]))
+        # Guard against calling tasks with no context.
+        if not isinstance(args[0], Context):
+            err = "Task expected a Context as its first arg, got {0} instead!"
+            # TODO: raise a custom subclass _of_ TypeError instead
+            raise TypeError(err.format(type(args[0])))
         result = self.body(*args, **kwargs)
         self.times_called += 1
         return result
@@ -122,7 +128,8 @@ class Task(object):
             * I.e. we *cannot* simply use a dict's ``keys()`` method here.
 
         * Second item is dict mapping arg names to default values or
-          task.NO_DEFAULT (i.e. an 'empty' value distinct from None).
+          `.NO_DEFAULT` (an 'empty' value distinct from None, since None
+          is a valid value on its own).
         """
         # Handle callable-but-not-function objects
         # TODO: __call__ exhibits the 'self' arg; do we manually nix 1st result
@@ -132,10 +139,13 @@ class Task(object):
         arg_names = spec.args[:]
         matched_args = [reversed(x) for x in [spec.args, spec.defaults or []]]
         spec_dict = dict(zip_longest(*matched_args, fillvalue=NO_DEFAULT))
-        # Remove context argument, if applicable
-        if self.contextualized:
+        # Pop context argument
+        try:
             context_arg = arg_names.pop(0)
-            del spec_dict[context_arg]
+        except IndexError:
+            # TODO: see TODO under __call__, this should be same type
+            raise TypeError("Tasks must have an initial Context argument!")
+        del spec_dict[context_arg]
         return arg_names, spec_dict
 
     def fill_implicit_positionals(self, positional):
@@ -160,7 +170,7 @@ class Task(object):
         # and move the underscored version to be the attr_name instead.)
         if '_' in name:
             opts['attr_name'] = name
-            name = name.replace('_', '-')
+            name = translate_underscores(name)
         names = [name]
         if self.auto_shortflags:
             # Must know what short names are available
@@ -172,7 +182,12 @@ class Task(object):
         # Handle default value & kind if possible
         if default not in (None, NO_DEFAULT):
             # TODO: allow setting 'kind' explicitly.
-            opts['kind'] = type(default)
+            # NOTE: skip setting 'kind' if optional is True + type(default) is
+            # bool; that results in a nonsensical Argument which gives the
+            # parser grief in a few ways.
+            kind = type(default)
+            if not (opts['optional'] and kind is bool):
+                opts['kind'] = kind
             opts['default'] = default
         # Help
         if name in self.help:
@@ -223,9 +238,6 @@ def task(*args, **kwargs):
     * ``name``: Default name to use when binding to a `.Collection`. Useful for
       avoiding Python namespace issues (i.e. when the desired CLI level name
       can't or shouldn't be used as the Python level name.)
-    * ``contextualized``: Hints to callers (especially the CLI) that this task
-      expects to be given a `~invoke.context.Context` object as its first
-      argument when called.
     * ``aliases``: Specify one or more aliases for this task, allowing it to be
       invoked as multiple different names. For example, a task named ``mytask``
       with a simple ``@task`` wrapper may only be invoked as ``"mytask"``.
@@ -248,27 +260,29 @@ def task(*args, **kwargs):
       flags from task options; defaults to True.
     * ``help``: Dict mapping argument names to their help strings. Will be
       displayed in ``--help`` output.
-    * ``pre``: List of task objects to execute prior to the
-      wrapped task whenever it is executed.
+    * ``pre``, ``post``: Lists of task objects to execute prior to, or after,
+      the wrapped task whenever it is executed.
+    * ``autoprint``: Boolean determining whether to automatically print this
+      task's return value to standard output when invoked directly via the CLI.
+      Defaults to False.
 
     If any non-keyword arguments are given, they are taken as the value of the
     ``pre`` kwarg for convenience's sake. (It is an error to give both
     ``*args`` and ``pre`` at the same time.)
     """
     # @task -- no options were (probably) given.
-    # Also handles ctask's use case when given as @ctask, equivalent to
-    # @task(obj, contextualized=True).
     if len(args) == 1 and callable(args[0]) and not isinstance(args[0], Task):
         return Task(args[0], **kwargs)
     # @task(pre, tasks, here)
     if args:
         if 'pre' in kwargs:
-            raise TypeError("May not give *args and 'pre' kwarg simultaneously!")
+            raise TypeError(
+                "May not give *args and 'pre' kwarg simultaneously!"
+            )
         kwargs['pre'] = args
     # @task(options)
     # TODO: pull in centrally defined defaults here (see Task)
     name = kwargs.pop('name', None)
-    contextualized = kwargs.pop('contextualized', False)
     aliases = kwargs.pop('aliases', ())
     positional = kwargs.pop('positional', None)
     optional = tuple(kwargs.pop('optional', ()))
@@ -277,15 +291,15 @@ def task(*args, **kwargs):
     help = kwargs.pop('help', {})
     pre = kwargs.pop('pre', [])
     post = kwargs.pop('post', [])
+    autoprint = kwargs.pop('autoprint', False)
     # Handle unknown kwargs
     if kwargs:
-        kwarg = (" unknown kwargs %r" % (kwargs,)) if kwargs else ""
+        kwarg = (" unknown kwargs {0!r}".format(kwargs)) if kwargs else ""
         raise TypeError("@task was called with" + kwarg)
     def inner(obj):
         obj = Task(
             obj,
             name=name,
-            contextualized=contextualized,
             aliases=aliases,
             positional=positional,
             optional=optional,
@@ -294,21 +308,12 @@ def task(*args, **kwargs):
             help=help,
             pre=pre,
             post=post,
+            autoprint=autoprint,
         )
         return obj
     return inner
 
 
-def ctask(*args, **kwargs):
-    """
-    Wrapper for `.task` which sets ``contextualized=True`` by default.
-
-    Please see `.task` for documentation.
-    """
-    kwargs.setdefault('contextualized', True)
-    return task(*args, **kwargs)
-
-
 class Call(object):
     """
     Represents a call/execution of a `.Task` with some arguments.
@@ -316,17 +321,117 @@ class Call(object):
     Wraps its `.Task` so it can be treated as one by `.Executor`.
 
     Similar to `~functools.partial` with some added functionality (such as the
-    delegation to the inner task, and optional tracking of a given name.)
+    delegation to the inner task, and optional tracking of the name it's being
+    called by.
     """
-    def __init__(self, task, *args, **kwargs):
+    def __init__(
+        self,
+        task,
+        called_as=None,
+        args=None,
+        kwargs=None,
+        context=None,
+    ):
+        """
+        Create a new `.Call` object.
+
+        :param task: The `.Task` object to be executed.
+
+        :param str called_as:
+            The name the task is being called as, e.g. if it was called by an
+            alias or other rebinding. Defaults to ``None``, aka, the task was
+            referred to by its default name.
+
+        :param tuple args:
+            Positional arguments to call with, if any. Default: ``None``.
+
+        :param dict kwargs:
+            Keyword arguments to call with, if any. Default: ``None``.
+
+        :param context:
+            `.Context` instance to be used. Default: ``None``.
+        """
         self.task = task
-        self.args = args
-        self.kwargs = kwargs
-        self.name = None # Mostly manipulated by client code
+        self.called_as = called_as
+        self.args = args or tuple()
+        self.kwargs = kwargs or dict()
+        self.context = context
 
     def __getattr__(self, name):
         return getattr(self.task, name)
 
+    def __deepcopy__(self, memo):
+        return self.clone()
 
-# Convenience/aesthetically pleasing-ish alias
-call = Call
+    def __str__(self):
+        aka = ""
+        if self.called_as is not None and self.called_as != self.task.name:
+            aka = " (called as: {0!r})".format(self.called_as)
+        return "<Call {0!r}{1}, args: {2!r}, kwargs: {3!r}>".format(
+            self.task.name, aka, self.args, self.kwargs
+        )
+
+    def __repr__(self):
+        return str(self)
+
+    def __eq__(self, other):
+        # NOTE: Not comparing 'called_as'; a named call of a given Task with
+        # same args/kwargs should be considered same as an unnamed call of the
+        # same Task with the same args/kwargs (e.g. pre/post task specified w/o
+        # name). Ditto tasks with multiple aliases.
+        for attr in "task args kwargs".split():
+            if getattr(self, attr) != getattr(other, attr):
+                return False
+        return True
+
+    def clone(self):
+        """
+        Return a standalone copy of this Call.
+
+        Useful when parameterizing task executions.
+        """
+        context = None
+        if self.context is not None:
+            # TODO: context.clone()?
+            context = Context(config=self.context.config.clone())
+        return Call(
+            task=self.task,
+            called_as=self.called_as,
+            args=deepcopy(self.args),
+            kwargs=deepcopy(self.kwargs),
+            context=context
+        )
+
+
+def call(task, *args, **kwargs):
+    """
+    Describes execution of a `.Task`, typically with pre-supplied arguments.
+
+    Useful for setting up :ref:`pre/post task invocations
+    <parameterizing-pre-post-tasks>`. It's actually just a convenient wrapper
+    around the `.Call` class, which may be used directly instead if desired.
+
+    For example, here's two build-like tasks that both refer to a ``setup``
+    pre-task, one with no baked-in argument values (and thus no need to use
+    `.call`), and one that toggles a boolean flag::
+
+        @task
+        def setup(ctx, clean=False):
+            if clean:
+                ctx.run("rm -rf target")
+            # ... setup things here ...
+            ctx.run("tar czvf target.tgz target")
+
+        @task(pre=[setup])
+        def build(ctx):
+            ctx.run("build, accounting for leftover files...")
+
+        @task(pre=[call(setup, clean=True)])
+        def clean_build(ctx):
+            ctx.run("build, assuming clean slate...")
+
+    Please see the constructor docs for `.Call` for details - this function's
+    ``args`` and ``kwargs`` map directly to the same arguments as in that
+    method.
+    """
+    return Call(task=task, args=args, kwargs=kwargs)
