@@ -2,32 +2,53 @@ from invoke.tasks import task
 
 
 @task(aliases=('bar', 'otherbar'))
-def foo():
+def foo(ctx):
     """
     Foo the bar.
     """
     pass
 
-@task(default=True)
-def biz():
-    pass
+@task
+def foo2(ctx):
+    """
+    Foo the bar:
 
-@task(help={'why': 'Motive', 'who': 'Who to punch'})
-def punch(who, why):
-    pass
+      example code
 
-@task(positional=['pos'])
-def one_positional(pos, nonpos):
-    pass
-
-@task(positional=['pos1', 'pos2'])
-def two_positionals(pos1, pos2, nonpos):
+    Added in 1.0
+    """
     pass
 
 @task
-def implicit_positionals(pos1, pos2, nonpos=None):
+def foo3(ctx):
+    """Foo the other bar:
+
+      example code
+
+    Added in 1.1
+    """
+    pass
+
+@task(default=True)
+def biz(ctx):
+    pass
+
+@task(help={'why': 'Motive', 'who': 'Who to punch'})
+def punch(ctx, who, why):
+    pass
+
+@task(positional=['pos'])
+def one_positional(ctx, pos, nonpos):
+    pass
+
+@task(positional=['pos1', 'pos2'])
+def two_positionals(ctx, pos1, pos2, nonpos):
+    pass
+
+@task
+def implicit_positionals(ctx, pos1, pos2, nonpos=None):
     pass
 
 @task(optional=['myopt'])
-def optional_values(myopt):
+def optional_values(ctx, myopt):
     pass
