@@ -1,21 +1,20 @@
 import operator
-import sys
 
-from spec import Spec, skip, eq_, raises, assert_raises
+from spec import Spec, eq_, ok_, raises, assert_raises
 
 from invoke.collection import Collection
 from invoke.tasks import task, Task
 from invoke.vendor import six
 from invoke.vendor.six.moves import reduce
 
-from _utils import load, support_path
+from _util import load, support_path
 
 
 @task
-def _mytask():
+def _mytask(ctx):
     six.print_("woo!")
 
-def _func():
+def _func(ctx):
     pass
 
 
@@ -25,10 +24,10 @@ class Collection_(Spec):
         def can_accept_task_varargs(self):
             "can accept tasks as *args"
             @task
-            def task1():
+            def task1(ctx):
                 pass
             @task
-            def task2():
+            def task2(ctx):
                 pass
             c = Collection(task1, task2)
             assert 'task1' in c
@@ -42,7 +41,7 @@ class Collection_(Spec):
         def kwargs_act_as_name_args_for_given_objects(self):
             sub = Collection()
             @task
-            def task1():
+            def task1(ctx):
                 pass
             ns = Collection(loltask=task1, notsub=sub)
             eq_(ns['loltask'], task1)
@@ -54,12 +53,11 @@ class Collection_(Spec):
             eq_(ns.collections['sub'], sub)
 
         def initial_string_arg_meshes_with_varargs_and_kwargs(self):
-            # Collection('myname', atask, acollection, othertask=taskobj, ...)
             @task
-            def task1():
+            def task1(ctx):
                 pass
             @task
-            def task2():
+            def task2(ctx):
                 pass
             sub = Collection('sub')
             ns = Collection('root', task1, sub, sometask=task2)
@@ -71,13 +69,17 @@ class Collection_(Spec):
             ):
                 eq_(x, y)
 
+        def accepts_load_path_kwarg(self):
+            eq_(Collection().loaded_from, None)
+            eq_(Collection(loaded_from='a/path').loaded_from, 'a/path')
+
     class useful_special_methods:
         def _meh(self):
             @task
-            def task1():
+            def task1(ctx):
                 pass
             @task
-            def task2():
+            def task2(ctx):
                 pass
             return Collection('meh', task1=task1, task2=task2)
 
@@ -163,13 +165,20 @@ class Collection_(Spec):
         class explicit_root_ns:
             def setup(self):
                 mod = load('explicit_root')
-                mod.ns.configure({'key': 'builtin', 'otherkey': 'yup'})
+                mod.ns.configure({
+                    'key': 'builtin',
+                    'otherkey': 'yup',
+                    'subconfig': {'mykey': 'myvalue'}
+                })
                 mod.ns.name = 'builtin_name'
                 self.unchanged = Collection.from_module(mod)
                 self.changed = Collection.from_module(
                     mod,
                     name='override_name',
-                    config={'key': 'override'}
+                    config={
+                        'key': 'override',
+                        'subconfig': {'myotherkey': 'myothervalue'}
+                    }
                 )
 
             def inline_config_with_root_namespaces_overrides_builtin(self):
@@ -177,7 +186,13 @@ class Collection_(Spec):
                 eq_(self.changed.configuration()['key'], 'override')
 
             def inline_config_overrides_via_merge_not_replacement(self):
-                assert 'otherkey' in self.changed.configuration()
+                ok_('otherkey' in self.changed.configuration())
+
+            def config_override_merges_recursively(self):
+                eq_(
+                    self.changed.configuration()['subconfig']['mykey'],
+                    'myvalue'
+                )
 
             def inline_name_overrides_root_namespace_object_name(self):
                 eq_(self.unchanged.name, 'builtin_name')
@@ -188,7 +203,6 @@ class Collection_(Spec):
                 # I.e. proves that the name doesn't end up 'explicit_root'.
                 eq_(self.unchanged.name, 'builtin_name')
 
-
     class add_task:
         def setup(self):
             self.c = Collection()
@@ -237,7 +251,7 @@ class Collection_(Spec):
 
         def specifying_default_False_overrides_task_setting(self):
             @task(default=True)
-            def its_me():
+            def its_me(ctx):
                 pass
             self.c.add_task(its_me, default=False)
             eq_(self.c.default, None)
@@ -319,13 +333,13 @@ class Collection_(Spec):
     class to_contexts:
         def setup(self):
             @task
-            def mytask(text, boolean=False, number=5):
+            def mytask(ctx, text, boolean=False, number=5):
                 six.print_(text)
             @task(aliases=['mytask27'])
-            def mytask2():
+            def mytask2(ctx):
                 pass
             @task(aliases=['othertask'], default=True)
-            def subtask():
+            def subtask(ctx):
                 pass
             sub = Collection('sub', subtask)
             self.c = Collection(mytask, mytask2, sub)
@@ -344,7 +358,7 @@ class Collection_(Spec):
 
         def positional_arglist_preserves_order_given(self):
             @task(positional=('second', 'first'))
-            def mytask(first, second, third):
+            def mytask(ctx, first, second, third):
                 pass
             c = Collection()
             c.add_task(mytask)
@@ -368,7 +382,10 @@ class Collection_(Spec):
             self.c = Collection.from_module(load('explicit_root'))
 
         def returns_all_task_names_including_subtasks(self):
-            eq_(set(self.c.task_names.keys()), set(['top_level', 'sub.sub_task']))
+            eq_(
+                set(self.c.task_names.keys()),
+                set(['top_level', 'sub.sub_task'])
+            )
 
         def includes_aliases_and_defaults_as_values(self):
             names = self.c.task_names
@@ -391,6 +408,13 @@ class Collection_(Spec):
             self.root.configure({'biz': 'baz'})
             eq_(set(self.root.configuration().keys()), set(['foo', 'biz']))
 
+        def configure_merging_is_recursive_for_nested_dicts(self):
+            self.root.configure({'foo': 'bar', 'biz': {'baz': 'boz'}})
+            self.root.configure({'biz': {'otherbaz': 'otherboz'}})
+            c = self.root.configuration()
+            eq_(c['biz']['baz'], 'boz')
+            eq_(c['biz']['otherbaz'], 'otherboz')
+
         def configure_allows_overwriting(self):
             self.root.configure({'foo': 'one'})
             eq_(self.root.configuration()['foo'], 'one')
