@@ -1,12 +1,10 @@
-from spec import Spec, eq_, skip, trap
-from mock import Mock, call as mock_call
+from spec import eq_, ok_
+from mock import Mock
 
-from invoke.context import Context
-from invoke.executor import Executor
-from invoke.collection import Collection
-from invoke.tasks import Task, ctask, call
+from invoke import Collection, Config, Context, Executor, Task, call, task
+from invoke.parser import ParserContext
 
-from _utils import _output_eq, IntegrationSpec
+from _util import expect, IntegrationSpec
 
 
 class Executor_(IntegrationSpec):
@@ -17,25 +15,34 @@ class Executor_(IntegrationSpec):
         self.task2 = Task(Mock(return_value=10), pre=[self.task1])
         self.task3 = Task(Mock(), pre=[self.task1])
         self.task4 = Task(Mock(return_value=15), post=[self.task1])
+        self.contextualized = Task(Mock())
         coll = Collection()
         coll.add_task(self.task1, name='task1')
         coll.add_task(self.task2, name='task2')
         coll.add_task(self.task3, name='task3')
         coll.add_task(self.task4, name='task4')
-        self.executor = Executor(collection=coll, context=Context())
+        coll.add_task(self.contextualized, name='contextualized')
+        self.executor = Executor(collection=coll)
 
     class init:
         "__init__"
-        def allows_collection_and_context(self):
+        def allows_collection_and_config(self):
             coll = Collection()
-            cont = Context()
-            e = Executor(collection=coll, context=cont)
+            conf = Config()
+            e = Executor(collection=coll, config=conf)
             assert e.collection is coll
-            assert e.context is cont
+            assert e.config is conf
 
-        def uses_blank_context_by_default(self):
+        def uses_blank_config_by_default(self):
             e = Executor(collection=Collection())
-            assert isinstance(e.context, Context)
+            assert isinstance(e.config, Config)
+
+        def can_grant_access_to_core_arg_parse_result(self):
+            c = ParserContext()
+            ok_(Executor(collection=Collection(), core=c).core is c)
+
+        def core_arg_parse_result_defaults_to_None(self):
+            ok_(Executor(collection=Collection()).core is None)
 
     class execute:
         def base_case(self):
@@ -45,7 +52,29 @@ class Executor_(IntegrationSpec):
         def kwargs(self):
             k = {'foo': 'bar'}
             self.executor.execute(('task1', k))
-            self.task1.body.assert_called_once_with(**k)
+            args = self.task1.body.call_args[0]
+            kwargs = self.task1.body.call_args[1]
+            ok_(isinstance(args[0], Context))
+            eq_(len(args), 1)
+            eq_(kwargs['foo'], 'bar')
+
+        def contextualized_tasks_are_given_parser_context_arg(self):
+            self.executor.execute('contextualized')
+            args = self.contextualized.body.call_args[0]
+            eq_(len(args), 1)
+            ok_(isinstance(args[0], Context))
+
+        def default_tasks_called_when_no_tasks_specified(self):
+            # NOTE: when no tasks AND no default, Program will print global
+            # help. We just won't do anything at all, which is fine for now.
+            task = Task(Mock('default-task'))
+            coll = Collection()
+            coll.add_task(task, name='mytask', default=True)
+            executor = Executor(collection=coll)
+            executor.execute()
+            args = task.body.call_args[0]
+            ok_(isinstance(args[0], Context))
+            eq_(len(args), 1)
 
     class basic_pre_post:
         "basic pre/post task functionality"
@@ -63,52 +92,42 @@ class Executor_(IntegrationSpec):
             t1 = Task(pre_body)
             t2 = Task(post_body)
             t3 = Task(Mock(), pre=[t1], post=[t2])
-            e = Executor(
-                collection=Collection(t1=t1, t2=t2, t3=t3),
-                context=Context()
-            )
+            e = Executor(collection=Collection(t1=t1, t2=t2, t3=t3))
             e.execute(('t3', {'something': 'meh'}))
             for body in (pre_body, post_body):
-                eq_(body.call_args, tuple())
+                args = body.call_args[0]
+                eq_(len(args), 1)
+                ok_(isinstance(args[0], Context))
 
-        def _call_objs(self, contextualized):
+        def _call_objs(self):
             # Setup
             pre_body, post_body = Mock(), Mock()
-            t1 = Task(pre_body, contextualized=contextualized)
-            t2 = Task(post_body, contextualized=contextualized)
+            t1 = Task(pre_body)
+            t2 = Task(post_body)
             t3 = Task(Mock(),
                 pre=[call(t1, 5, foo='bar')],
                 post=[call(t2, 7, biz='baz')],
             )
             c = Collection(t1=t1, t2=t2, t3=t3)
-            e = Executor(collection=c, context=Context())
+            e = Executor(collection=c)
             e.execute('t3')
             # Pre-task asserts
             args, kwargs = pre_body.call_args
             eq_(kwargs, {'foo': 'bar'})
-            if contextualized:
-                assert isinstance(args[0], Context)
-                eq_(args[1], 5)
-            else:
-                eq_(args, (5,))
+            assert isinstance(args[0], Context)
+            eq_(args[1], 5)
             # Post-task asserts
             args, kwargs = post_body.call_args
             eq_(kwargs, {'biz': 'baz'})
-            if contextualized:
-                assert isinstance(args[0], Context)
-                eq_(args[1], 7)
-            else:
-                eq_(args, (7,))
-
-        def may_be_call_objects_specifying_args(self):
-            self._call_objs(False)
+            assert isinstance(args[0], Context)
+            eq_(args[1], 7)
 
         def call_objs_play_well_with_context_args(self):
-            self._call_objs(True)
+            self._call_objs()
 
     class deduping_and_chaining:
         def chaining_is_depth_first(self):
-            _output_eq('-c depth_first deploy', """
+            expect('-c depth_first deploy', out="""
 Cleaning HTML
 Cleaning .tar.gz files
 Cleaned everything
@@ -120,7 +139,7 @@ Testing
 """.lstrip())
 
         def _expect(self, args, expected):
-            _output_eq('-c integration {0}'.format(args), expected.lstrip())
+            expect('-c integration {0}'.format(args), out=expected.lstrip())
 
         class adjacent_hooks:
             def deduping(self):
@@ -200,47 +219,51 @@ bar
             pre = [call(t1, 5), call(t1, 7), call(t1, 5)]
             t2 = Task(Mock(), pre=pre)
             c = Collection(t1=t1, t2=t2)
-            e = Executor(collection=c, context=Context())
+            e = Executor(collection=c)
             e.execute('t2')
             # Does not call the second t1(5)
-            body.assert_has_calls([mock_call(5), mock_call(7)])
+            param_list = []
+            for body_call in body.call_args_list:
+                ok_(isinstance(body_call[0][0], Context))
+                param_list.append(body_call[0][1])
+            ok_(set(param_list) == set((5, 7)))
 
-    class configuration:
+    class collection_driven_config:
+        "Collection-driven config concerns"
         def hands_collection_configuration_to_context(self):
-            @ctask
+            @task
             def mytask(ctx):
-                eq_(ctx['my.config.key'], 'value')
+                eq_(ctx.my_key, 'value')
             c = Collection(mytask)
-            c.configure({'my.config.key': 'value'})
-            Executor(collection=c, context=Context()).execute('mytask')
+            c.configure({'my_key': 'value'})
+            Executor(collection=c).execute('mytask')
 
         def hands_task_specific_configuration_to_context(self):
-            @ctask
+            @task
             def mytask(ctx):
-                eq_(ctx['my.config.key'], 'value')
-            @ctask
+                eq_(ctx.my_key, 'value')
+            @task
             def othertask(ctx):
-                eq_(ctx['my.config.key'], 'othervalue')
+                eq_(ctx.my_key, 'othervalue')
             inner1 = Collection('inner1', mytask)
-            inner1.configure({'my.config.key': 'value'})
+            inner1.configure({'my_key': 'value'})
             inner2 = Collection('inner2', othertask)
-            inner2.configure({'my.config.key': 'othervalue'})
+            inner2.configure({'my_key': 'othervalue'})
             c = Collection(inner1, inner2)
-            e = Executor(collection=c, context=Context())
+            e = Executor(collection=c)
             e.execute('inner1.mytask', 'inner2.othertask')
 
         def subcollection_config_works_with_default_tasks(self):
-            @ctask(default=True)
+            @task(default=True)
             def mytask(ctx):
-                eq_(ctx['my.config.key'], 'value')
-            # Sets up a task "known as" sub.mytask which may be called as just
-            # 'sub' due to being default.
+                eq_(ctx.my_key, 'value')
+            # Sets up a task "known as" sub.mytask which may be called as
+            # just 'sub' due to being default.
             sub = Collection('sub', mytask=mytask)
-            sub.configure({'my.config.key': 'value'})
+            sub.configure({'my_key': 'value'})
             main = Collection(sub=sub)
             # Execute via collection default 'task' name.
-            Executor(collection=main, context=Context()).execute('sub')
-
+            Executor(collection=main).execute('sub')
 
     class returns_return_value_of_specified_task:
         def base_case(self):
@@ -257,3 +280,19 @@ bar
                 self.executor.execute('task4'),
                 {self.task1: 7, self.task4: 15}
             )
+
+    class autoprinting:
+        def defaults_to_off_and_no_output(self):
+            expect("-c autoprint nope", out="")
+
+        def prints_return_value_to_stdout_when_on(self):
+            expect("-c autoprint yup", out="It's alive!\n")
+
+        def prints_return_value_to_stdout_when_on_and_in_collection(self):
+            expect("-c autoprint sub.yup", out="It's alive!\n")
+
+        def does_not_fire_on_pre_tasks(self):
+            expect("-c autoprint pre_check", out="")
+
+        def does_not_fire_on_post_tasks(self):
+            expect("-c autoprint post_check", out="")
