@@ -2,13 +2,13 @@ import imp
 import os
 import sys
 
-from spec import Spec, skip, eq_, raises
+from spec import Spec, eq_, raises
 
 from invoke.loader import Loader, FilesystemLoader as FSLoader
 from invoke.collection import Collection
 from invoke.exceptions import CollectionNotFound
 
-from _utils import support
+from _util import support
 
 
 class _BasicLoader(Loader):
@@ -28,6 +28,13 @@ class Loader_(Spec):
         # Crummy doesn't-explode test.
         _BasicLoader().load('namespacing')
 
+    def doesnt_dupliate_parent_dir_addition(self):
+        _BasicLoader().load('namespacing')
+        _BasicLoader().load('namespacing')
+        # If the bug is present, this will be 2 at least (and often more, since
+        # other tests will pollute it (!).
+        eq_(sys.path.count(support), 1)
+
     def closes_opened_file_object(self):
         loader = _BasicLoader()
         loader.load('foo')
@@ -73,5 +80,6 @@ class FilesystemLoader_(Spec):
 
     def defaults_to_tasks_collection(self):
         "defaults to 'tasks' collection"
-        result = FSLoader(start=support + '/implicit/').load()
+        # There's a basic tasks.py in tests/_support
+        result = self.l.load()
         eq_(type(result), Collection)
