@@ -1,6 +1,6 @@
 import copy
 
-from spec import Spec, eq_, skip, ok_, raises
+from spec import Spec, eq_, skip, raises
 
 from invoke.parser import Argument, Context
 from invoke.tasks import task
@@ -97,7 +97,7 @@ class Context_(Spec):
             # here. Yes that makes this an integration test, but it's nice to
             # test it here at this level & not just in cli tests.
             @task
-            def mytask(underscored_option=True):
+            def mytask(ctx, underscored_option=True):
                 pass
             self.c.add_arg(mytask.get_arguments()[0])
             eq_(
@@ -165,7 +165,7 @@ class Context_(Spec):
             # Task/Collection generated Context
             # (will expose flags n such)
             @task(help={'otherarg': 'other help'}, optional=['optval'])
-            def mytask(myarg, otherarg, optval):
+            def mytask(ctx, myarg, otherarg, optval):
                 pass
             col = Collection(mytask)
             self.tasked = col.to_contexts()[0]
@@ -227,7 +227,14 @@ class Context_(Spec):
 
         def underscored_args(self):
             c = Context(args=(Argument('i_have_underscores', help='yup'),))
-            eq_(c.help_for('--i-have-underscores'), ('--i-have-underscores=STRING', 'yup'))
+            eq_(
+                c.help_for('--i-have-underscores'),
+                ('--i-have-underscores=STRING', 'yup')
+            )
+
+        def true_default_args(self):
+            c = Context(args=(Argument('truthy', kind=bool, default=True),))
+            eq_(c.help_for('--truthy'), ('--[no-]truthy', ''))
 
     class help_tuples:
         def returns_list_of_help_tuples(self):
@@ -235,7 +242,7 @@ class Context_(Spec):
             # TODO: consider redoing help_for to be more flexible on input --
             # arg value or flag; or even Argument objects. ?
             @task(help={'otherarg': 'other help'})
-            def mytask(myarg, otherarg):
+            def mytask(ctx, myarg, otherarg):
                 pass
             c = Collection(mytask).to_contexts()[0]
             eq_(
@@ -278,7 +285,7 @@ class Context_(Spec):
                 ['--longarg', '--otherarg']
             )
 
-        def sorts_heterogenous_help_output_with_longflag_only_options_first(self):
+        def sorts_heterogenous_help_output_with_longflag_only_options_first(self): # noqa
             # When both of the above mix, long-flag-only options come first.
             # E.g.:
             #   --alpha
@@ -314,12 +321,12 @@ class Context_(Spec):
     class str:
         "__str__"
         def with_no_args_output_is_simple(self):
-            eq_(str(Context('foo')), "<Context 'foo'>")
+            eq_(str(Context('foo')), "<parser/Context 'foo'>")
 
         def args_show_as_repr(self):
             eq_(
                 str(Context('bar', args=[Argument('arg1')])),
-                "<Context 'bar': {'arg1': <Argument: arg1>}>"
+                "<parser/Context 'bar': {'arg1': <Argument: arg1>}>"
             )
 
         def repr_is_str(self):
